﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using net.pdnyet.odbcproxy;
using System.ServiceModel;
using System.ServiceModel.Web;
using System.Data.Odbc;
using System.Data.OleDb;
using System.IO.Compression;
using System.Data.Common;

namespace net.pdynet.odbcproxy
{
    [ServiceBehavior(InstanceContextMode = InstanceContextMode.PerCall,
        ConcurrencyMode = ConcurrencyMode.Multiple)]
    class OdbcProxyService : IOdbcProxyService
    {

        #region IOdbcProxyService Members

        public ConnectResponse Connect(ConnectRequest request)
        {
            //WebOperationContext ctx = WebOperationContext.Current;
            //ctx.OutgoingResponse.StatusCode = System.Net.HttpStatusCode.NoContent;

            ConnectResponse response = new ConnectResponse();
            
            // Overeni, zda byl zadan Connection string;
            if (StringUtils.IsBlank(request.ConnectionString))
            {
                response.Error = "Blank connection string.";
            }
            else
            {
                // Otevreni ODBC.
                try
                {
                    PooledOdbcConnection pooledOdbcConnection = OdbcConnectionPool.Instance.OdbcConnect(request.ConnectionString, request.UsingOleDb);
                    response.ConnectionID = pooledOdbcConnection.ID;
                    response.Success = true;
                    response.ConnectionAutoCloseTime = pooledOdbcConnection.ConnectionAutoCloseTime;
                }
                catch (OdbcException x)
                {
                    response.Error = OdbcConnectionPool.GetOdbcError(x);
                }
                catch (Exception x)
                {
                    response.Error = x.ToString();
                }
            }

            return response;
        }

        public CloseResponse Close(CloseRequest request)
        {
            CloseResponse response = new CloseResponse();

            if (StringUtils.IsBlank(request.ConnectionID))
            {
                response.Error = "Blank connection ID.";
            }
            else
            {
                try
                {
                    OdbcConnectionPool.Instance.CloseConnection(request.ConnectionID);
                    response.Success = true;
                }
                catch (OdbcException x)
                {
                    response.Error = OdbcConnectionPool.GetOdbcError(x);
                }
                catch (Exception x)
                {
                    response.Error = x.ToString();
                }
            }

            return response;
        }

        public SelectResponse Select(SelectRequest request)
        {
            SelectResponse response = new SelectResponse();

            if (StringUtils.IsBlank(request.ConnectionID))
            {
                response.Error = "Blank connection ID.";
            }
            else if (StringUtils.IsBlank(request.Query))
            {
                response.Error = "Query is blank.";
            }
            else
            {
                try
                {
                    PooledOdbcConnection pooledOdbcConnection = OdbcConnectionPool.Instance.GetConnection(request.ConnectionID);

                    List<RowData> rows = new List<RowData>();
                    response.Rows = rows;

                    if (pooledOdbcConnection.OdbcConnection != null)
                    {
                        OdbcConnection connection = pooledOdbcConnection.OdbcConnection;

                        using (OdbcCommand command = new OdbcCommand(request.Query, connection))
                        {
                            using (OdbcDataReader reader = command.ExecuteReader())
                            {
                                while (reader.Read())
                                {
                                    RowData rowData = new RowData();
                                    for (int i = 0; i < reader.FieldCount; i++)
                                    {
                                        rowData.Add(reader.GetName(i), getColumnValue(reader, i));
                                    }
                                    rows.Add(rowData);
                                }
                            }
                        }
                    }
                    else
                    {
                        OleDbConnection connection = pooledOdbcConnection.OleDbConnection;

                        using (OleDbCommand command = new OleDbCommand(request.Query, connection))
                        {
                            using (OleDbDataReader reader = command.ExecuteReader())
                            {
                                while (reader.Read())
                                {
                                    RowData rowData = new RowData();
                                    for (int i = 0; i < reader.FieldCount; i++)
                                    {
                                        rowData.Add(reader.GetName(i), getColumnValue(reader, i));
                                    }
                                    rows.Add(rowData);
                                }
                            }
                        }
                    }

                    response.Success = true;
                    response.ConnectionAutoCloseTime = pooledOdbcConnection.ConnectionAutoCloseTime;
                }
                catch (OdbcException x)
                {
                    response.Error = OdbcConnectionPool.GetOdbcError(x);
                }
                catch (Exception x)
                {
                    response.Error = x.ToString();
                }
            }

            return response;
        }

        public ExecuteResponse ExecuteCommand(ExecuteRequest request)
        {
            ExecuteResponse response = new ExecuteResponse();

            if (StringUtils.IsBlank(request.ConnectionID))
            {
                response.Error = "Blank connection ID.";
            }
            else if (StringUtils.IsBlank(request.Query))
            {
                response.Error = "Query is blank.";
            }
            else
            {
                //Console.WriteLine(request.Query);
                try
                {
                    PooledOdbcConnection pooledOdbcConnection = OdbcConnectionPool.Instance.GetConnection(request.ConnectionID);

                    if (pooledOdbcConnection.OdbcConnection != null)
                    {
                        OdbcConnection connection = pooledOdbcConnection.OdbcConnection;

                        using (OdbcCommand command = new OdbcCommand(request.Query, connection))
                        {
                            int affectedRows = command.ExecuteNonQuery();
                            response.AffectedRows = affectedRows;
                        }
                    }
                    else
                    {
                        OleDbConnection connection = pooledOdbcConnection.OleDbConnection;

                        using (OleDbCommand command = new OleDbCommand(request.Query, connection))
                        {
                            int affectedRows = command.ExecuteNonQuery();
                            response.AffectedRows = affectedRows;
                        }
                    }

                    response.Success = true;
                    response.ConnectionAutoCloseTime = pooledOdbcConnection.ConnectionAutoCloseTime;
                }
                catch (OdbcException x)
                {
                    response.Error = OdbcConnectionPool.GetOdbcError(x);
                }
                catch (Exception x)
                {
                    response.Error = x.ToString();
                }
            }

            return response;
        }

        #endregion

        private object getColumnValue(DbDataReader reader, int index)
        {
            object result = null;

            if (!reader.IsDBNull(index))
            {
                //reader.GetFieldType();
                //result = reader.GetValue(index).ToString();
                result = reader.GetValue(index);
            }

            return result;
        }

        /*
        protected void CompressResponseStream(WebOperationContext context)
        {
            if (context == null)
                context = WebOperationContext.Current;

            string encodings = context.IncomingRequest.Headers.Get("Accept-Encoding");

            if (!String.IsNullOrEmpty(encodings))
            {
                encodings = encodings.ToLowerInvariant();

                if (encodings.Contains("deflate"))
                {
                    //context.OutgoingRequest.
                    context.Response.Filter = new DeflateStream(context.Response.Filter, CompressionMode.Compress);
                    context.Response.AppendHeader("Content-Encoding", "deflate");
                    context.Response.AppendHeader("X-CompressResponseStream", "deflate");
                }
                else if (encodings.Contains("gzip"))
                {
                    context.Response.Filter = new GZipStream(context.Response.Filter, CompressionMode.Compress);
                    context.Response.AppendHeader("Content-Encoding", "gzip");
                    context.Response.AppendHeader("X-CompressResponseStream", "gzip");
                }
                else
                {
                    context.Response.AppendHeader("X-CompressResponseStream", "no-known-accept");
                }
            }
        }
        */
    }
}
